local PetBattle = select(2, ...)
local L, MAX_PETS, MAX_ABILITIES = PetBattle.L, PetBattle.MAX_PETS, PetBattle.MAX_ABILITIES

local Abilities = PetBattle:RegisterModule("Abilities")

-- Update the UI
function Abilities:Update(battlePets)
	if( not PetBattle.db.showAbilities ) then return end

	local allyType = C_PetBattles.GetPetType(LE_BATTLE_PET_ALLY, C_PetBattles.GetActivePet(LE_BATTLE_PET_ALLY))

	local frame = self.frame
	frame:SetHeight(20 + (table.getn(battlePets[LE_BATTLE_PET_ENEMY]) * (80 + 25)))

	for slot, pet in pairs(battlePets[LE_BATTLE_PET_ENEMY]) do
		local row = frame.rows[slot]
		row.petOwner = LE_BATTLE_PET_ENEMY
		row.petSlot = slot

		row.petType:SetTexture("Interface\\PetBattles\\PetIcon-" .. PET_TYPE_SUFFIX[pet["petType"]])
		row.icon:SetTexture(pet["icon"])

		-- Pet's dead, gray out their info to make it obvious
		if( pet["health"] <= 0 ) then
			row.dead = true
			row.icon:SetDesaturated(true)
			row.icon:SetVertexColor(0.5, 0.5, 0.5)

			row.petType:SetDesaturated(true)
			row.petType:SetVertexColor(0.5, 0.5, 0.5)

			row.name.fs:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b)

		-- Still alive, still aliveeee
		else
			if( row.dead ) then
				row.dead = nil
				row.icon:SetDesaturated(false)
				row.icon:SetVertexColor(1, 1, 1)

				row.petType:SetDesaturated(false)
				row.petType:SetVertexColor(1, 1, 1)
			end

			local color = ITEM_QUALITY_COLORS[pet["quality"] - 1]
			row.name.fs:SetTextColor(color.r, color.g, color.b)
		end

		-- Now name
		if( not row.dead and pet["active"] ) then
			row.name:SetFormattedText("%s%s%s %s-%s %s[%s]%s %s", NORMAL_FONT_COLOR_CODE, L["A"], FONT_COLOR_CODE_CLOSE, HIGHLIGHT_FONT_COLOR_CODE, FONT_COLOR_CODE_CLOSE, NORMAL_FONT_COLOR_CODE, pet["level"], FONT_COLOR_CODE_CLOSE, pet["name"])
		else
			row.name:SetFormattedText(L["%s[%s]%s %s"], NORMAL_FONT_COLOR_CODE, pet["level"], FONT_COLOR_CODE_CLOSE, pet["name"])
		end

		-- Now all the fun abilities
		for abilitySlot, ability in pairs(pet["abilities"]) do
			local abilityRow = row.abilities[abilitySlot]
			abilityRow.icon:SetTexture(ability["icon"])
			
			abilityRow.petType:SetTexture("Interface\\PetBattles\\PetIcon-" .. PET_TYPE_SUFFIX[ability["petType"]])
			
			-- Flags for the ability tooltips
			abilityRow.petOwner = LE_BATTLE_PET_ENEMY
			abilityRow.petSlot = slot
			abilityRow.petAbilitySlot = abilitySlot

			-- Weak/strong to the attack
			if( ability["hints"] ) then
				local modifier = C_PetBattles.GetAttackModifier(ability["petType"], allyType)
				if( modifier > 1 ) then
					abilityRow.modifierIcon:SetTexture("Interface\\PetBattles\\BattleBar-AbilityBadge-Strong")
					abilityRow.modifierIcon:Show()

				elseif( modifier < 0 ) then
					abilityRow.modifierIcon:SetTexture("Interface\\PetBattles\\BattleBar-AbilityBadge-Weak")
					abilityRow.modifierIcon:Show()
				else
					abilityRow.modifierIcon:Hide()
				end
			else
				abilityRow.modifierIcon:Hide()
			end

			-- Cooldowns
			if( ability["cooldown"] and ability["cooldown"] > 0 ) then
				abilityRow.icon:SetVertexColor(0.5, 0.5, 0.5)
				abilityRow.icon:SetDesaturated(true)

				abilityRow.cooldown:SetText(ability["cooldown"])
				abilityRow.cooldown:Show()

				abilityRow.shadow:Show()

		    elseif( not ability["usable"] )	 then
				abilityRow.icon:SetVertexColor(0.5, 0.5, 0.5)
				abilityRow.icon:SetDesaturated(true)

				abilityRow.cooldown:Hide()
				abilityRow.shadow:Hide()

			else
				abilityRow.icon:SetVertexColor(1, 1, 1)
				abilityRow.icon:SetDesaturated(false)
				abilityRow.cooldown:Hide()
				abilityRow.shadow:Hide()
     	    end

			abilityRow.name:SetText(ability["name"])

			-- Check for any useful to know info like stuns/sleeps/etc
     	    local tag;
     	    if( tag ) then
     	    	abilityRow.tag:SetText(tag)
     	    else
     	    	abilityRow.tag:Hide()
     	    end

     	    -- Gray out everything if the pet is dead
     	    if( pet["health"] <= 0 ) then
				abilityRow.icon:SetVertexColor(0.5, 0.5, 0.5)
				abilityRow.icon:SetDesaturated(true)

				abilityRow.petType:SetVertexColor(0.5, 0.5, 0.5)
				abilityRow.petType:SetDesaturated(true)

				abilityRow.modifierIcon:SetVertexColor(0.5, 0.5, 0.5)
				abilityRow.modifierIcon:SetDesaturated(true)

				abilityRow.name:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b)
				abilityRow.tag:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b)

				abilityRow.dead = true

			-- Need to reset states
			elseif( abilityRow.dead ) then
				abilityRow.dead = nil

				abilityRow.petType:SetVertexColor(1, 1, 1)
				abilityRow.petType:SetDesaturated(false)

				abilityRow.modifierIcon:SetVertexColor(1, 1, 1)
				abilityRow.modifierIcon:SetDesaturated(false)

				abilityRow.name:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
				abilityRow.tag:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
     	    end

			abilityRow:Show()
		end

		row:Show()
	end
end

-- General management
function Abilities:SetStatus()
	if( PetBattle.db.showAbilities ) then
		self:Create()
	else
		self:Reset()
	end
end

function Abilities:Reset()
	if( not self.frame ) then return end

	for _, row in pairs(self.frame.rows) do
		for _, ability in pairs(row.abilities) do
			ability:Hide()
		end

		row:Hide()
	end

	self.frame:Hide()
end

-- UI Creation
local function ShowPetTooltip(self)
	local r, g, b = self.fs:GetTextColor()

	GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
	GameTooltip:AddLine(self:GetText(), r, g, b)
	GameTooltip:AddLine(L["Click to view in journal"])
	GameTooltip:Show()
end

function Abilities:Create()
	if( not PetBattle.db.showAbilities ) then return end
	if( self.frame ) then return self.frame:Show() end

	-- Container frame
	self.frame = CreateFrame("Frame", nil, UIParent)
	self.frame:SetWidth(365)
	self.frame:SetMovable(true)
	self.frame:EnableMouse(true)
	self.frame:SetClampedToScreen(true)
	
	self.frame:SetBackdrop({
		bgFile = "Interface\\CharacterFrame\\UI-Party-Background",
		edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
		tile = true,
		tileSize = 32,
		edgeSize = 32,
		insets = {left = 10, right = 10, top = 10, bottom = 10},
	})
	
	self.frame:SetBackdropColor(0, 0, 0, 0.90)

	-- Create the title/movy thing
	local texture = self.frame:CreateTexture(nil, "ARTWORK")
	texture:SetTexture("Interface\\DialogFrame\\UI-DialogBox-Header")
	texture:SetPoint("TOP", 0, 13)
	texture:SetWidth(240)
	texture:SetHeight(60)
	
	local title = CreateFrame("Button", nil, self.frame)
	title:SetPoint("TOP", 0, 4)
	title:SetText(L["Enemy Pets"])
	title:SetPushedTextOffset(0, 0)
	title:SetNormalFontObject(GameFontNormal)
	title:SetHeight(20)
	title:SetWidth(200)

	title:SetScript("OnEnter", function(self)
		GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
		GameTooltip:SetText(L["Alt + Left Click to drag"])
	end)
	title:SetScript("OnLeave", PetBattle.HideTooltip)

	title:RegisterForDrag("LeftButton")
	title:SetScript("OnDragStart", function(self)
		if( IsAltKeyDown() ) then
			self.isMoving = true
			self:GetParent():StartMoving()
		end
	end)
	
	title:SetScript("OnDragStop", function(self)
		if( not self.isMoving ) then return end

		self.isMoving = nil

		local parent = self:GetParent()
		parent:StopMovingOrSizing()

		--local scale = parent:GetEffectiveScale()
		local scale = 1
		PetBattle.db.abilityPosition = {x = parent:GetLeft() * scale, y = parent:GetTop() * scale}
	end)
	
	if( PetBattle.db.abilityPosition ) then
		--local scale = self.frame:GetEffectiveScale()
		local scale = 1
		self.frame:ClearAllPoints()
		self.frame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", PetBattle.db.abilityPosition.x / scale, PetBattle.db.abilityPosition.y / scale)
	else
		self.frame:SetPoint("CENTER")
	end

	self.frame.rows = {}

	for petSlot=1, MAX_PETS do
		local row = CreateFrame("Frame", nil, self.frame)
		row:SetHeight(80)
		row:Hide()

		row.icon = row:CreateTexture(nil, "ARTWORK")
		row.icon:SetPoint("TOPLEFT", row, "TOPLEFT")
		row.icon:SetSize(28, 28)

		row.petType = row:CreateTexture(nil, "ARTWORK")
		row.petType:SetPoint("TOPLEFT", row.icon, "TOPRIGHT", 8, 0)
		row.petType:SetTexCoord(0.79687500, 0.49218750, 0.50390625, 0.65625000)
		row.petType:SetSize(26, 26)

		row.name = CreateFrame("Button", nil, row)
		row.name:SetNormalFontObject(GameFontNormalLarge)
		row.name:SetHighlightFontObject(GameFontHighlightLarge)
		row.name:SetWidth(245)
		row.name:SetHeight(20)
		row.name:SetScript("OnEnter", function(self) PetBattle.ShowPetTooltip(self:GetParent(), self) end)
		row.name:SetScript("OnLeave", function(self) PetBattle.HidePetTooltip(self:GetParent(), self) end)
		row.name:SetPushedTextOffset(0, 0)
		row.name:SetPoint("TOPLEFT", row.petType, "TOPRIGHT", 14, -2)

		row.name:SetText("*")
		row.name.fs = row.name:GetFontString()
		row.name.fs:SetWordWrap(false)
		row.name.fs:SetWidth(row.name:GetWidth())
		row.name.fs:SetJustifyH("LEFT")

		row.abilities = {}

		for abilitySlot=1, MAX_ABILITIES do
			local ability = CreateFrame("Frame", nil, row)
			ability:SetScript("OnEnter", PetBattle.ShowAbilityTooltip)
			ability:SetScript("OnLeave", PetBattle.HideAbilityTooltip)
			ability:SetWidth(100)
			ability:SetHeight(50)
			ability:Hide()

			ability.icon = ability:CreateTexture(nil, "ARTWORK")
			ability.icon:SetPoint("TOPLEFT", ability, "TOPLEFT")
			ability.icon:SetSize(22, 22)

			ability.cooldown = ability:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge")
			ability.cooldown:SetPoint("CENTER", ability.icon)

 			ability.shadow = ability:CreateTexture(nil, "OVERLAY", "BattleBar-Countdown-Shadow")
			ability.shadow:ClearAllPoints()
			ability.shadow:SetSize(26, 26)
			ability.shadow:SetPoint("CENTER", ability.icon)

			ability.petType = ability:CreateTexture(nil, "ARTWORK")
			ability.petType:SetPoint("TOPLEFT", ability.icon, "TOPRIGHT", 8, 0)
			ability.petType:SetTexCoord(0.79687500, 0.49218750, 0.50390625, 0.65625000)
			ability.petType:SetSize(22, 22)

			ability.modifierIcon = ability:CreateTexture(nil, "ARTWORK")
			ability.modifierIcon:SetPoint("TOPLEFT", ability.petType, "TOPRIGHT", 0, 6)
			ability.modifierIcon:SetSize(34, 34)

			ability.name = ability:CreateFontString(nil, "OVERLAY", "GameFontNormal")
			ability.name:SetPoint("TOPLEFT", ability.icon, "BOTTOMLEFT", 0, -6)

			ability.tag = ability:CreateFontString(nil, "OVERLAY", "GameFontHighlight")
			ability.tag:SetPoint("TOPLEFT", ability.name, "BOTTOMLEFT", 0, -2)

			if( abilitySlot > 1 ) then
				ability:SetPoint("TOPLEFT", row.abilities[abilitySlot - 1], "TOPRIGHT", 14, 0)
			else
				ability:SetPoint("TOPLEFT", row, "TOPLEFT", 2, -36) 
			end

			row.abilities[abilitySlot] = ability
		end

		if( petSlot > 1 ) then
			row:SetPoint("TOPLEFT", self.frame.rows[petSlot - 1], "BOTTOMLEFT", 0, -20)
			row:SetPoint("TOPRIGHT", self.frame.rows[petSlot - 1], "BOTTOMRIGHT", 0, 0)
		else
			row:SetPoint("TOPLEFT", self.frame, "TOPLEFT", 16, -32)
			row:SetPoint("TOPRIGHT", self.frame, "TOPRIGHT")
		end

		self.frame.rows[petSlot] = row
	end


	self.frame:Show()
end